from bunnyhop import base


class Billing(base.BaseBunny):

    def get(self):
        return BillingSummary(self.api_key, **self.call_api(f"/billing", "GET"))

    def apply_code(self, coupon_code):
        return self.call_api(f"/billing/applycode?couponCode={coupon_code}", "GET")


class BillingRecord(base.BaseBunny):
    Id = base.IntegerProperty()
    PaymentId = base.CharProperty()
    Amount = base.FloatProperty()
    Payer = base.EmailProperty()
    Timestamp = base.DateTimeProperty()
    Type = base.IntegerProperty()
    InvoiceAvailable = base.BooleanProperty()

    def __str__(self):
        return f"{self.Id} - {self.Amount}"


class BillingSummary(base.BaseBunny):
    Balance = base.FloatProperty(required=True)
    ThisMonthCharges = base.FloatProperty(required=True)
    BillingRecords = base.ListProperty(BillingRecord)
    MonthlyChargesStorage = base.FloatProperty(required=True)
    MonthlyChargesEUTraffic = base.FloatProperty(required=True)
    MonthlyChargesUSTraffic = base.FloatProperty(required=True)
    MonthlyChargesASIATraffic = base.FloatProperty(required=True)
    MonthlyChargesSATraffic = base.FloatProperty(required=True)

    def __str__(self):
        return f"{self.Balance}"

    @property
    def billing_records(self):
        return [BillingRecord(self.api_key, **i) for i in self.BillingRecords]



from bunnyhop import base


class Zone(base.BaseBunny):
    Id = base.IntegerProperty()
    Name = base.CharProperty()
    OriginUrl = base.CharProperty()
    Enabled = base.BooleanProperty(default_value=False)
    Hostnames = base.ListProperty()
    StorageZoneId = base.IntegerProperty()
    AllowedReferrers = base.ListProperty()
    BlockedReferrers = base.ListProperty()
    BlockedIps = base.ListProperty()
    EnableGeoZoneUS = base.BooleanProperty(default_value=False)
    EnableGeoZoneEU = base.BooleanProperty(default_value=False)
    EnableGeoZoneASIA = base.BooleanProperty(default_value=False)
    EnableGeoZoneSA = base.BooleanProperty(default_value=False)
    EnableGeoZoneAF = base.BooleanProperty(default_value=False)
    ZoneSecurityEnabled = base.BooleanProperty(default_value=False)
    ZoneSecurityKey = base.CharProperty()
    ZoneSecurityIncludeHashRemoteIP = base.BooleanProperty(default_value=False)
    IgnoreQueryStrings = base.BooleanProperty(default_value=False)
    MonthlyBandwidthLimit = base.IntegerProperty()
    MonthlyBandwidthUsed = base.IntegerProperty()
    MonthlyCharges = base.FloatProperty()
    AddHostHeader = base.BooleanProperty(default_value=False)
    Type = base.IntegerProperty()
    CustomNginxConfig = base.CharProperty()
    AccessControlOriginHeaderExtensions = base.ListProperty()
    EnableAccessControlOriginHeader = base.BooleanProperty(default_value=False)
    DisableCookies = base.BooleanProperty(default_value=False)
    BudgetRedirectedCountries = base.ListProperty()
    BlockedCountries = base.ListProperty()
    EnableOriginShield = base.BooleanProperty(default_value=False)
    CacheControlMaxAgeOverride = base.IntegerProperty()
    CacheControlPublicMaxAgeOverride = base.IntegerProperty()
    BurstSize = base.IntegerProperty()
    RequestLimit = base.IntegerProperty()
    BlockRootPathAccess = base.BooleanProperty(default_value=False)
    BlockPostRequests = base.BooleanProperty(default_value=False)
    CacheQuality = base.IntegerProperty()
    LimitRatePerSecond = base.FloatProperty()
    LimitRateAfter = base.FloatProperty()
    ConnectionLimitPerIPCount = base.IntegerProperty()
    PriceOverride = base.FloatProperty()
    AddCanonicalHeader = base.BooleanProperty(default_value=False)
    EnableLogging = base.BooleanProperty(default_value=False)
    IgnoreVaryHeader = base.BooleanProperty(default_value=False)
    EnableCacheSlice = base.BooleanProperty(default_value=False)
    EdgeRules = base.ListProperty()
    EnableWebPVary = base.BooleanProperty(default_value=False)
    EnableCountryCodeVary = base.BooleanProperty(default_value=False)
    EnableMobileVary = base.BooleanProperty(default_value=False)
    EnableHostnameVary = base.BooleanProperty(default_value=False)
    CnameDomain = base.CharProperty()
    AWSSigningEnabled = base.BooleanProperty(default_value=False)
    AWSSigningKey = base.CharProperty()
    AWSSigningSecret = base.CharProperty()
    AWSSigningRegionName = base.CharProperty()
    LoggingIPAnonymizationEnabled = base.BooleanProperty(default_value=False)
    EnableTLS1 = base.BooleanProperty(default_value=False)
    EnableTLS1_1 = base.BooleanProperty(default_value=False)
    VerifyOriginSSL = base.BooleanProperty(default_value=False)
    OriginShieldZoneCode = base.CharProperty()

    def __str__(self):
        return self.Name

    @property
    def host_names(self):
        return [Hostname(self.api_key, **i) for i in self.Hostnames]

    def get(self, id):
        response = self.call_api(f"/pullzone/{id}", "GET")
        try:
            if response.get('Id', None):
                return Zone(self.api_key, **response)
        except:
            return "Zone not found."

    def create(self, Name, Type, OriginUrl, StorageZoneId=None):
        api_data = {
            'Name': Name,
            'Type': Type,
            'OriginUrl': OriginUrl,
            'StorageZoneId': StorageZoneId
        }
        response = self.call_api(f"/pullzone", "POST", self.get_header(),
                                 json_data=api_data)

        if response.get('Id', None):
            return Zone(self.api_key, **response)
        return response

    def list(self):
        return [Zone(self.api_key, **i) for i in self.call_api(f"/pullzone",
                                                               "GET")]

    def update(
            self,
            id,
            OriginUrl=None,
            AllowedReferrers=None,
            BlockedIps=None,
            EnableCacheSlice=None,
            EnableGeoZoneUS=None,
            EnableGeoZoneEU=None,
            EnableGeoZoneASIA=None,
            EnableGeoZoneSA=None,
            EnableGeoZoneAF=None,
            ZoneSecurityEnabled=None,
            ZoneSecurityIncludeHashRemoteIP=None,
            IgnoreQueryStrings=None,
            MonthlyBandwidthLimit=None,
            AccessControlOriginHeaderExtensions=None,
            EnableAccessControlOriginHeader=None,
            BlockRootPathAccess=None,
            EnableWebpVary=None,
            EnableLogging=None,
            DisableCookies=None,
            BudgetRedirectedCountries=None,
            BlockedCountries=None,
            EnableOriginShield=None,
            OriginShieldZoneCode=None,
            AddCanonicalHeader=None,
            CacheControlMaxAgeOverride=None,
            AddHostHeader=None,
            AWSSigningEnabled=None,
            AWSSigningKey=None,
            AWSSigningRegionName=None,
            AWSSigningSecret=None,
            EnableTLS1=None,
            EnableTLS1_1=None):
        api_data = {
            "OriginUrl": OriginUrl,
            "AllowedReferrers": AllowedReferrers,
            "BlockedIps": BlockedIps,
            "EnableCacheSlice": EnableCacheSlice,
            "EnableGeoZoneUS": EnableGeoZoneUS,
            "EnableGeoZoneEU": EnableGeoZoneEU,
            "EnableGeoZoneASIA": EnableGeoZoneASIA,
            "EnableGeoZoneSA": EnableGeoZoneSA,
            "EnableGeoZoneAF": EnableGeoZoneAF,
            "ZoneSecurityEnabled": ZoneSecurityEnabled,
            "ZoneSecurityIncludeHashRemoteIP": ZoneSecurityIncludeHashRemoteIP,
            "IgnoreQueryStrings": IgnoreQueryStrings,
            "MonthlyBandwidthLimit": MonthlyBandwidthLimit,
            "AccessControlOriginHeaderExtensions":
                AccessControlOriginHeaderExtensions,
            "EnableAccessControlOriginHeader": EnableAccessControlOriginHeader,
            "BlockRootPathAccess": BlockRootPathAccess,
            "EnableWebpVary": EnableWebpVary,
            "EnableLogging": EnableLogging,
            "DisableCookies": DisableCookies,
            "BudgetRedirectedCountries": BudgetRedirectedCountries,
            "BlockedCountries": BlockedCountries,
            "EnableOriginShield": EnableOriginShield,
            "OriginShieldZoneCode": OriginShieldZoneCode,
            "AddCanonicalHeader": AddCanonicalHeader,
            "CacheControlMaxAgeOverride": CacheControlMaxAgeOverride,
            "AddHostHeader": AddHostHeader,
            "AWSSigningEnabled": AWSSigningEnabled,
            "AWSSigningKey": AWSSigningKey,
            "AWSSigningRegionName": AWSSigningRegionName,
            "AWSSigningSecret": AWSSigningSecret,
            "EnableTLS1": EnableTLS1,
            "EnableTLS1_1": EnableTLS1_1
        }
        return self.call_api(f"/pullzone/{id}", "POST", self.get_header(),
                             json_data=api_data)

    def delete(self, id):
        response = self.call_api(f"/pullzone/{id}", "DELETE",
                                 self.get_header())
        if type(response) is bytes:
            return "Zone deleted."
        return response

    def purge(self, id):
        response = self.call_api(f"/pullzone/{id}/purgeCache", "POST",
                                 self.get_header())
        if type(response) is bytes:
            return "Purge Queue was finished."
        return response

    def purge_file(self, filename):
        url = (f"http://{self.host_names[0].Value}/{filename}")
        response = self.call_api(f"/purge", "POST", params={'url': url})
        if type(response) is bytes:
            return "Purge Queue was finished."
        return response

    def add_hostname(self, hostname):
        response = self.call_api(f"/pullzone/addHostname", "POST", self.get_header(),
                                 json_data={'PullZoneId': self.Id, 'Hostname': hostname})
        return response

    def delete_hostname(self, hostname):
        response = self.call_api(f"/pullzone/deleteHostname", "POST", self.get_header(),
                                 json_data={'PullZoneId': self.Id, 'Hostname': hostname})
        return response


    def create_edge_rule(
            self,
            id,
            Guid=None,
            ActionParameter1=None,
            ActionParameter2=None,
            Enabled=None,
            Description=None,
            ActionType=None,
            TriggerMatchingType=None,
            Triggers=None):
        api_data = {
            "Guid": Guid,
            "ActionParameter1": ActionParameter1,
            "ActionParameter2": ActionParameter2,
            "Enabled": Enabled,
            "Description": Description,
            "ActionType": ActionType,
            "TriggerMatchingType": TriggerMatchingType,
            "Triggers": Triggers
        }
        response = self.call_api(f"/pullzone/{id}/edgerules/addOrUpdate",
                                 "POST", self.get_header(), json_data=api_data)
        if type(response) is bytes:
            return"Successfully Added/Updated an edge rule."
        return response


class Hostname(base.BaseBunny):
    Id = base.IntegerProperty()
    Value = base.CharProperty()
    ForceSSL = base.BooleanProperty(default_value=False)
    IsSystemHostname = base.BooleanProperty(default_value=False)
    HasCertificate = base.BooleanProperty(default_value=False)

    def __str__(self):
        return f"{self.Value}"



from bunnyhop import base


class Stats(base.BaseBunny):
    TotalBandwidthUsed = base.IntegerProperty()
    TotalRequestsServed = base.IntegerProperty()
    CacheHitRate = base.FloatProperty()
    BandwidthUsedChart = base.DictProperty()
    BandwidthCachedChart = base.DictProperty()
    CacheHitRateChart = base.DictProperty()
    RequestsServedChart = base.DictProperty()
    PullRequestsPulledChart = base.DictProperty()
    OriginShieldBandwidthUsedChart = base.DictProperty()
    OriginShieldInternalBandwidthUsedChart = base.DictProperty()
    UserBalanceHistoryChart = base.DictProperty()
    GeoTrafficDistribution = base.DictProperty()
    Error3xxChart = base.DictProperty()
    Error4xxChart = base.DictProperty()
    Error5xxChart = base.DictProperty()

    def __str__(self):
        return f"Bandwidth: {self.TotalBandwidthUsed} | Total Requests: {self.TotalRequestsServed} | " \
               f"CacheHitRate: {self.CacheHitRate}"

    def get(self, date_from=None, date_to=None, pull_zone=None, server_zone_id=None):
        params = {
            'dateFrom': date_from,
            'dateTo': date_to,
            'pullZone': pull_zone,
            'serverZoneId': server_zone_id
        }
        return Stats(self.api_key, **self.call_api(
            f"/statistics?dateFrom", "GET", params=params))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['bunnyhop']

package_data = \
{'': ['*']}

install_requires = \
['envs>=1.3,<2.0', 'requests>=2.23.0,<3.0.0', 'valley>=1.5.5,<2.0.0']

setup_kwargs = {
    'name': 'bunnyhop',
    'version': '0.1.1',
    'description': 'A Python library created make building with BunnyCDN easier',
    'long_description': '# bunnyhop\nPython library for BunnyCDN\n\n## Status\nDocumentation only phase\n\n## Install\n\n```shell script\npip install bunnyhop\n```\n\n## Getting Started \n\n```python\nfrom bunnyhop import Bunny\n\nb = Bunny(\'<api_key>\')\n```\n## Pull Zones\n\n#### Get a Pull Zone\n\n```python\nzone = b.Zone.get(id=\'myzone\')\n#Returns: <Zone: myzone > \n```\n\n#### Create a Pull Zone\n```python\nb.Zone.create(\n    Name=\'myzone\',\n    Type=0, # 0 = Standard and 1 = High Volume\n    OriginUrl=\'https://example.org\',\n    StorageZoneId=\'storage-zone-id\'\n)\n```\n\n#### List Pull Zones\n```python\nb.Zone.list()\n```\n\n#### Update a Pull Zone\n```python\nb.Zone.update(\n    id=\'zone-id\', #Positional Argument\n    OriginUrl=\'http://example.org\',\n    AllowedReferrers=[\'app.example.org\', \'www.example.org\'],\n    BlockedIps=[\'122.33.22.11\'],\n    EnableCacheSlice=False,\n    EnableGeoZoneUS=True,\n    EnableGeoZoneEU=True,\n    EnableGeoZoneASIA=True,\n    EnableGeoZoneSA=True,\n    EnableGeoZoneAF=True,\n    ZoneSecurityEnabled=True,\n    ZoneSecurityIncludeHashRemoteIP=True,\n    IgnoreQueryStrings=True,\n    MonthlyBandwidthLimit=1073741824,\n    AccessControlOriginHeaderExtensions=[\'jpg\', \'png\'],\n    EnableAccessControlOriginHeader=True,\n    BlockRootPathAccess=True,\n    EnableWebpVary=True,\n    EnableLogging=True,\n    DisableCookies=False,\n    BudgetRedirectedCountries=[\'RU\', \'BR\'],\n    BlockedCountries=[\'RU\', \'BR\'],\n    EnableOriginShield=True,\n    OriginShieldZoneCode=\'FR\',\n    AddCanonicalHeader=0,\n    CacheControlMaxAgeOverride=-1,\n    AddHostHeader=True,\n    AWSSigningEnabled=True,\n    AWSSigningKey=\'AK_EXAMPLEKEY\',\n    AWSSigningRegionName=\'us-east-1\',\n    AWSSigningSecret=\'SK_EXAMPLESECRETKET\',\n    EnableTLS1=True,\n    EnableTLS1_1=True\n    \n)\n# From Zone object\nzone.update(\n    OriginUrl=\'http://example.org\',\n    AllowedReferrers=[\'app.example.org\', \'www.example.org\'],\n    BlockedIps=[\'122.33.22.11\'],\n    EnableCacheSlice=False,\n    EnableGeoZoneUS=True,\n    EnableGeoZoneEU=True,\n    EnableGeoZoneASIA=True,\n    EnableGeoZoneSA=True,\n    EnableGeoZoneAF=True,\n    ZoneSecurityEnabled=True,\n    ZoneSecurityIncludeHashRemoteIP=True,\n    IgnoreQueryStrings=True,\n    MonthlyBandwidthLimit=1073741824,\n    AccessControlOriginHeaderExtensions=[\'jpg\', \'png\'],\n    EnableAccessControlOriginHeader=True,\n    BlockRootPathAccess=True,\n    EnableWebpVary=True,\n    EnableLogging=True,\n    DisableCookies=False,\n    BudgetRedirectedCountries=[\'RU\', \'BR\'],\n    BlockedCountries=[\'RU\', \'BR\'],\n    EnableOriginShield=True,\n    OriginShieldZoneCode=\'FR\',\n    AddCanonicalHeader=0,\n    CacheControlMaxAgeOverride=-1,\n    AddHostHeader=True,\n    AWSSigningEnabled=True,\n    AWSSigningKey=\'AK_EXAMPLEKEY\',\n    AWSSigningRegionName=\'us-east-1\',\n    AWSSigningSecret=\'SK_EXAMPLESECRETKET\',\n    EnableTLS1=True,\n    EnableTLS1_1=True \n)\n```\n\n#### Delete Pull Zone\n```python\n#From top-level\nb.Zone.delete(id=\'myzone\')\n\n#From Zone object\nzone.delete()\n```\n\n#### Purge File From Pull Zone\n```python\n# From top-level\nb.Purge.create(url=\'https://myzone.b-cdn.net/css/style.css\')\n\n# From Zone Object\nzone.purge_file(\'css/style.css\')\n```\n\n#### Purge Esntire Pull Zone\n```python\n# From top-level\nb.Zone.purge(\n    id=\'myzone\'\n)\n# From Zone object\nzone.purge()\n```\n\n#### Add Edge Rule to Pull Zone\n```python\nb.Zone.create_edge_rule(\n    id=\'myzone\',\n    Guid="6a2e94df-8aa9-4cd2-b89d-16752102ef9f", # GUID of the edge rule\n    ActionParameter1 = "My-Header",\n    ActionParameter2 = "HeaderValue",\n    Enabled = True,\n    Description = "My header value",\n    ActionType = 0, # 0 = ForceSSL, 1 = Redirect, 2 = OriginUrl, 3 = OverrideCacheTime, 4 = BlockRequest, 5 = SetResponseHeader, 6 = SetRequestHeader, 7 = ForceDownload, 8 = DisableTokenAuthentication, 9 = EnableTokenAuthentication\n    TriggerMatchingType = 1    \n    Triggers = []\n)\n```\n## Storage\n\n### Storage Zones\n\n#### Create Storage Zone\n```python\n\n# Create a Storage Zone (available regions are listed below)\nb.Storage.create(name=\'example-a\', main_storage_region=\'NY\', replica_regions=[\'DE\', \'SG\', \'SYD\'])\n# Returns: <StorageZone: example-a (id: 1234)>\n```\n\n##### Storage Regions\n\n- DE - Europe (Falkenstein)\n- NY - North America (New York)\n- SG - Asia (Singapore)\n- SYD - Oceania (Sydney)\n\n#### List Storage Zones\n```python\nb.Storage.all()\n# Returns: [<StorageZone: example-a (id: 1234)>, <StorageZone: example-b (id: 12345)>]\n```\n\n#### Delete a Storage Zone\n```python\nb.Storage.delete(1234)\n# Returns: None\n```\n\n#### Get a Storage Zone\n```python\nzone = b.Storage.get(1234)\n# Returns: <StorageZone: example-a (id: 1234)>\n```\n\n### Storage Files\n\n```python\n# Get a Storage Zone\nzone = b.StorageZone.get(1234)\n# Returns: <StorageZone: example-a (id: 1234)>\n\n# Lists files in the zone\nobj_list = zone.all()\n# Returns [<StorageFile: index.html>, <StorageFile: base.css>]\n\n# Get a file\nobj = zone.get(\'index.html\')\n#Returns <StorageFile: index.html>\n\n# Download that File\nobj.download()\n\n# Delete a file\nzone.delete(\'index.html\')\n\n# Upload a File\nzone.upload_file(dest_path=\'folder/path/error.html\', local_path=\'/home/mj/work/error.html\')\n# Returns: <StorageFile: error.html>\n\n# Create a file from a string\nzone.create_file(\'base.css\', "body {background-color: powderblue;}")\n# Returns: <StorageFile: base.css>\n```\n\n### Storage JSON Files\n\n```python\n\n# Create a json file\nmj = zone.create_json(\'23.json\', {\'first_name\':\'Michael\', \'last_name\': \'Jordan\'})\n# Returns <StorageJSONFile: 23.json>\n\n# Get information from file\nmj[\'first_name\']\n# Returns: \'Michael\'\n```\n\n# Purge\n\n## Create a Purge\n```python\n# Purge a zone or file\nb.Purge.create(url=\'https://myzone.b-cdn.net/style.css\')\n```\n\n# Stats\n\n```python\nb.Stats.get(dateFrom=\'2018-12-01\', dateTo=\'2020-01-01\', pullZone=\'example-zone\', serverZoneId=\'serverZoneID\')\n```\n# Billing\n\n#### Get Billing Summary\n```python\nb.Billing.get()\n```\n\n#### ApplyCode\n\n```python\nb.Billing.applycode(couponCode=\'somecode123\')\n```\n',
    'author': 'Brian Jinwright',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/capless/bunnyhop',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
